/* 
 * econtacts.cpp
 * 
 * proram to manage contact emails;
 *   names and emails of folks.
 */

#include <cstring>
#include <cstdlib>
#include <cstdio>
#include <cctype>

// we need a structure (class) to store the actual
// data in...
class EContact {
    // private data of the class.
    char* name;
    char* email;
public:

    // default constructor
    EContact(){
        EContact("","");
    }

    // copy constructor.
    //   contact = contact 
    EContact(const EContact& c){
        // allocate memory for name.
        name = new char[ strlen(c.name)+1 ];
        strcpy(name,c.name);
        email = new char[ strlen(c.email)+1 ];
        strcpy(email,c.email);
    }

    // construct an object from name, and email.
    EContact(const char* n,const char* e){
        // allocate memory for name.
        name = new char[ strlen(n)+1 ];
        strcpy(name,n);
        email = new char[ strlen(e)+1 ];
        strcpy(email,e);
    }

    ~EContact(){
        delete[] name;
        delete[] email;
    }
    char* getName(){
        return name;
    }
    char* getEmail(){
        return email;
    }
    void setName(const char* s){
        delete[] name;
        name = new char[ strlen(s)+1 ];
        strcpy(name,s);
    }
    void setEmail(const char* s){
        delete[] email;
        email = new char[ strlen(s)+1 ];
        strcpy(email,s);
    }
    // assignment (e.g. = `operator');
    void set(const EContact& c){
        setName(c.name);
        setEmail(c.name);
    }
    EContact& operator=(const EContact& c){
        setName(c.name);
        setEmail(c.name);
        return *this;
    }
};

// remove trailing space (new lines, tabs, etc.) characters.
char* trim(char* s){
    int len = strlen(s);
    while(len > 0 && isspace(s[len-1])){
        s[len-1] = '\0';
        len--;
    }
    return s;
}

// prompt a user with str, if user types in anything,
// return that, else, return d (default).
char* prompt(const char* str,const char* d,char* mem,int maxlen){
    printf("%s",str);
    if(d != 0)
        printf(" [%s]",d);
    printf(": ");
    if(fgets(mem,maxlen,stdin) == 0)
        return (char*)d;
    int len = strlen(mem);  // get length of entered str.
    if(len <= 1)
        return (char*)d;
    /*
    if(mem[len-1] == '\n')  // if last character is newline
        mem[len-1] = '\0';  // clear it out.
    */
    return trim(mem);
}




EContact readContact(EContact* d){
    char name[100];
    char email[100];

    char* n = prompt("Name",d ? d->getName():NULL,name,sizeof(name));
    char* e = prompt("Email",d ? d->getEmail():NULL,email,sizeof(email));
    
    return EContact(n,e);
}

// display a contact with a number (index of contact).
void writeContact(EContact& c,int n){
    printf("%d,%s,%s\n",n,c.getName(),
        c.getEmail());    
}

// our program to read commands from user and 
// peform them. commands will include:
/// add
/// remove
/// list / sort
/// edit
/// find
int main(){
    int i;
    EContact* allcontacts = new EContact[0];
    int numallcontacts = 0;

    char buf[1000];     // command buffer.
    printf("> ");
    // read next line (command) until end of file.
    while(fgets(buf,sizeof(buf),stdin)){
        trim(buf);
        //printf("you typed: [%s]\n",buf);

        // if user wants to get out... break this loop.
        if(!strcmp(buf,"exit") || !strcmp(buf,"quit"))
            break;

        // user wants to add a contact.
        if(!strcmp(buf,"add")){
            
            // read a contact...
            EContact c = readContact(NULL);
            
            // note, this contact is scoped 
            // to this if statement, once
            // it leaves this if statement,
            // the destructor gets called, 
            // and name/email are deleted.

            //printf("adding: %s, %s\n",c.getName(),c.getEmail());

            // add contact to "storage" array.
            // allocate a new array with size + 1
            EContact* p = new EContact[numallcontacts+1];
            // copy all contacts.
            for(i=0;i<numallcontacts;i++)
                p[i].set(allcontacts[i]);
            // last contact is the one we just read.
            p[numallcontacts].set(c);

            //printf("add-ed: %s, %s\n",p[0].getName(),p[0].getEmail());
            
            numallcontacts++;       // add contact.
            delete[] allcontacts;   // delete old storage.
            allcontacts = p;        // use new storage.
        }

        if(!strcmp(buf,"list")){
            for(i=0;i<numallcontacts;i++)
                writeContact(allcontacts[i],i);
        }


        if(!strcmp(buf,"remove")){
            char *p = prompt("Id of contact to remove",NULL,buf,sizeof(buf));
            if(p && isdigit(p[0])){
                int id = atoi(p);
                if(id >= 0 && id < numallcontacts){
                    writeContact(allcontacts[id],id);
                    prompt("Erase (y/n)","n",buf,sizeof(buf));
                    if(tolower(buf[0]) == 'y'){

                        // delete contact with id...
                        // put the last contact in the place we want to delete... and decrease the counter.
                        allcontacts[id].set(allcontacts[numallcontacts-1]);
numallcontacts--;

                    }
                }
            }
        }



        if(!strcmp(buf,"edit")){
            char *p = prompt("Id of contact to edit",NULL,buf,sizeof(buf));
            if(p && isdigit(p[0])){
                int id = atoi(p);
                if(id >= 0 && id < numallcontacts){
                    writeContact(allcontacts[id],id);
                    EContact c = readContact(&allcontacts[id]);
                    writeContact(c,id);
                    prompt("Save Changes? (y/n)","n",buf,sizeof(buf));
                    if(tolower(buf[0]) == 'y'){

                        // delete contact with id...
                        allcontacts[id].set(c);
                    }
                }
            }
        }


        if(!strcmp(buf,"find")){
            char *p = prompt("search string",NULL,buf,sizeof(buf));
            if(p){
                // lowercase everything.
                for(i=0;i<strlen(p);i++)
                    p[i] = tolower(p[i]);
                
                for(i=0;i<numallcontacts;i++){
                    char buf2[1000];
                    strcpy(buf2,allcontacts[i].getName());
                    strcat(buf2,allcontacts[i].getEmail());
                    for(int j=0;buf2[j];j++)
                        buf2[j] = tolower(buf2[j]);

                    // printf("lookin for %s in %s\n",p,buf2);
                    if(strstr(buf2,p)) 
                    writeContact(allcontacts[i],i);
                }
            }
        }

        printf("> ");
    }

    return 0;
}



